package org.shadoware.eggskeeper.engine;

import java.util.HashMap;
import java.util.Map;


/**
 * Manages all the animations of an object
 * 
 * @author dhenar
 *
 */
public class Animator {

	private final Map<String, Animation> animations;
	
	private long initTime;
	private long nowTime;
	
	
	public Animator() {
		
		animations = new HashMap<String, Animation>();
	}
	
	
	public void update(String animationName) {
		
		updateAnimation(animations.get(animationName));
	}
	
	
	public int getFrame(String animationName) {
		
		return animations.get(animationName).getSequence();
	}
	
	
	/**
	 * Update the given animation
	 */
	private void updateAnimation(Animation animation) {
		
		nowTime = System.nanoTime() / 1000000;
		
		if (nowTime > initTime + animation.getTime()) {
			
			initTime = nowTime;
			animation.next();
		}
	}
	
	
	/**
	 * Adds a new animation
	 */
	public void addAnimation(String name, int[] sequence, int[] time) {
		
		animations.put(name, new Animation(name, sequence, time));
	}
	
	
	/**
	 * Remove an animation
	 */
	public void removeAnimation(String name) {
		
		animations.remove(name);
	}
}


/**
 * Single animation sequence
 * 
 * @author dhenar
 *
 */
class Animation {
	
	private final int[] 	sequence;
	private final int[] 	time;
	private final String 	name;
	
	private int position;
	
	
	/**
	 * The sequence has the same length that time. Time and sequence MUST be the 
	 * same lenght;
	 * 
	 * @param sequence : frame numbers
	 * @param time : values in milleseconds
	 */
	public Animation(String name, int[] sequence, int[] time) {
		
		this.sequence 	= sequence;
		this.time		= time;
		this.name		= name;
	}

	
	/**
	 * Gets the current frame sequence
	 */
	public int getSequence() {
		
		return sequence[position];
	}
	
	
	/**
	 * Gets the current frame time;
	 */
	public int getTime() {
		
		return time[position];
	}
	
	
	/**
	 * Gets the name of the animation
	 */
	public String getName() {
		
		return name;
	}	
	
	
	/**
	 * Go to the next frame
	 */
	public void next() {
		
		position = ( position + 1) % sequence.length;
	}
	
	
	/**
	 * Reset the position to the first frame
	 */
	public void reset() {
		
		position = 0;
	}
	
}
