package org.shadoware.eggskeeper.engine;

import java.util.ArrayList;
import java.util.List;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import org.shadoware.eggskeeper.engine.math.ColorRGBA;
import org.shadoware.eggskeeper.engine.sound.Sounds;
import org.shadoware.eggskeeper.engine.util.FPSCounter;

import android.content.Context;
import android.opengl.GLES20;
import android.opengl.GLSurfaceView;
import android.opengl.Matrix;


/**
 * It renders the scene
 * 
 * @author David Henar
 *
 */
public class Scene implements GLSurfaceView.Renderer {
	
	private final int 	FAR_PLANE 			= 100;			// far plane distance
	private final float 	CAMERA_INIT_HEIGHT 	= 5.0f; 		// distance from the plane to the eye
	
	private final float[] mModelMatrix 		= new float[16];		// transformations
	private final float[] mViewMatrix 		= new float[16];		// camera
	private final float[] mProjectionMatrix 	= new float[16];		// projections
	private final float[] mMVPMatrix 			= new float[16];		// combination of all
	
	private final Camera2D camera;
    private final ColorRGBA bgColor;
    private final List<SceneElement> elements = new ArrayList<SceneElement>();
    
    protected Context context;
    
    private FPSCounter 	mFpsCounter;
    
    private TouchControl controllerPad;
    
    
    /**
     * Basic black back background renderer
     */
    public Scene(Context context, int width, int height) {
    	this.context 		= context;
    	this.bgColor 		= new ColorRGBA();
    	this.camera 	 	= new Camera2D(CAMERA_INIT_HEIGHT);
    	this.mFpsCounter 	= new FPSCounter();
    	this.controllerPad = new TouchControl();
    }
    
    
    /**
     * Colored background renderer
     */
    public Scene(Context context, int width, int height, ColorRGBA bgColor) {
    	this(context, width, height);
    	this.bgColor.set(bgColor);
    }
    

    /**
     * Main iteration loops
     */
    public void onDrawFrame(GL10 gl) {
    	
    	clearBackground();
        updateFPS();
        
        for (SceneElement element : elements) {
        	
        	if ( element instanceof Updatable ) {
        		((Updatable) element).update();
        	}
        	
        	if (element instanceof LiveSceneElement) {
        		((LiveSceneElement)element).getBody().update(mFpsCounter.getTimeInc());
        	}
        	
        	renderElement(element);
        }        
        
        renderHUD(camera.getHud());
    }
    
    
    private void updateFPS() {
    	
    	mFpsCounter.update();
    }
    
    
    protected void renderElement(SceneElement renderable) {
    	
    	renderable.transform(mModelMatrix);
    	updateMVP(mModelMatrix, mViewMatrix, mProjectionMatrix, mMVPMatrix);
    	renderable.render(mMVPMatrix);
    }
    
    
    /**
     * Draw background color
     */
    private void clearBackground() {
    	
        GLES20.glClear(GLES20.GL_COLOR_BUFFER_BIT);
    }
    
    
    private void renderHUD(HUD hud) {
    	
    	for (HUDElement element : hud.getElements()) {
    		renderElement(element.getRenderable());
    	}
    }

    
    public void onSurfaceChanged(GL10 gl, int width, int height) {
    	
    	GLES20.glViewport(0, 0, width, height);
        updateProjection(mProjectionMatrix, width, height);
    }

    
    public void onSurfaceCreated(GL10 gl, EGLConfig config) {	
    	
    	reset(gl);
    }
    
    
    private void reset(GL10 gl) {
    	
    	Sounds.getInstance().initSounds(context);
    	Textures.getInstance().clearAllTextures();
    	Shaders.getInstance().loadShaders(context.getAssets());
    	glClearColor(bgColor);  
    	setTransparency(gl, true);
    	camera.update(mViewMatrix);
    }
    
    
    /**
     * Sets a color into the render clear color
     */
    private void glClearColor(ColorRGBA c) {
    	
    	GLES20.glClearColor(
    			c.get(ColorRGBA.R), 
    			c.get(ColorRGBA.G), 
    			c.get(ColorRGBA.B), 
    			c.get(ColorRGBA.A)
    	);
    }

    
    /**
     * visible scene elements
     */
	public List<SceneElement> getElements() {
		
		return elements;
	}
	
	
	/**
	 * Calculate the Model-View-Projection matrix to pass this data into the vertex shader.
	 * 
	 * @param mModelMatrix - transformations data
	 * @param mViewMatrix	- camera data
	 * @param mProjectionMatrix	 - projection data
	 * @param mMVPMatrix - prepared data to render
	 */
	private void updateMVP( 
			float[] mModelMatrix, 
			float[] mViewMatrix, 
			float[] mProjectionMatrix,
			float[] mMVPMatrix) {
		
		// combine the model with the view matrix
		Matrix.multiplyMM(mMVPMatrix, 0, mViewMatrix, 		0, mModelMatrix, 0);
		
		// combine the model-view with the projection matrix
        Matrix.multiplyMM(mMVPMatrix, 0, mProjectionMatrix, 0, mMVPMatrix, 	 0);
	}
    
	
	/**
	 * this projection matrix is applied to object coordinates.
	 * 
	 * @param mProjectionMatrix
	 */
	private void updateProjection(float[] mProjectionMatrix, float width, float height) {
		
    	float ratio = (float) width / height;
        //Matrix.frustumM(mProjectionMatrix, 0, -ratio, ratio, -1, 1, 1, FAR_PLANE);
    	Matrix.orthoM(mProjectionMatrix, 0, -ratio, ratio, -1, 1, 1, FAR_PLANE);
	}
	
	
	/**
	 * Sets the global transparency.
	 * 
	 * @param gl
	 * @param transparency - true to work with alpha textures
	 */
	private void setTransparency(GL10 gl, boolean transparency) {
		
		if (transparency) {
			gl.glEnable(GL10.GL_BLEND);
			gl.glBlendFunc(GL10.GL_ONE, GL10.GL_ONE_MINUS_SRC_ALPHA);
		}
		else {
			gl.glDisable(GL10.GL_BLEND);
		}
	}

	
	public TouchControl getControllerPad() {
		
		return controllerPad;
	}


	public void setControllerPad(TouchControl controllerPad) {
		
		this.controllerPad = controllerPad;
	}


	/**
	 * Gets the camera of the scene, this allows to change it's patameters
	 */
	public Camera2D getCamera() {
		
		return camera;
	}


	/**
	 * Gets the background color of the scene, this allows to change it's RGB values.
	 * The alpha chanel does not affect in this case, It still opaque allways.
	 */
	public ColorRGBA getBgColor() {
		
		return bgColor;
	}
	
}
