package org.shadoware.eggskeeper.engine;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;
import java.nio.ShortBuffer;

import org.shadoware.eggskeeper.engine.math.ColorRGBA;
import org.shadoware.eggskeeper.engine.math.Tuple3f;

import android.opengl.GLES20;
import android.opengl.Matrix;


/**
 * Basic element in the scene... a simple square.
 * 
 * @author David Henar
 */
public class SceneElement implements Renderable {
	
	private final String DEF_PROGRAM = "basic";
	
	private final FloatBuffer vertexBuffer;
    private final ShortBuffer drawListBuffer;
    
	private final int COORDS_PER_VERTEX = 3;
	private final int vertexStride = COORDS_PER_VERTEX * 4; // 4 bytes per vertex
	
    private final float squareCoords[] = { 
    	-0.5f,  0.5f, 0.0f,  	// top left
    	-0.5f, -0.5f, 0.0f,   	// bottom left
         0.5f, -0.5f, 0.0f,   	// bottom right
         0.5f,  0.5f, 0.0f }; 	// top right

    private final short drawOrder[] = { 0, 1, 2, 0, 2, 3 }; // order to draw vertices

    protected final TextureFrames textureFrames;
    protected 	int activeFrame;

    // Accessible attributes
    protected final 	ColorRGBA 	color;
    protected final	Tuple3f		position;
    protected final	Tuple3f 	rotation;
    protected 			float 		size;
    protected			boolean 	visible;
    
    
    // program and handlers
    protected int mProgram;
    protected int mTexture;
    
    
    /**
     * Base constructor
     */
    public SceneElement(int frames) {
    	
    	size 			= 1.0f;
    	color 			= new ColorRGBA(1.0f, 1.0f, 1.0f, 1.0f);	// white by default
    	position 		= new Tuple3f();
    	rotation		= new Tuple3f();
    	vertexBuffer   	= createVertexBuffer(squareCoords);
    	drawListBuffer 	= initializeDrawList(drawOrder);
    	textureFrames	= new TextureFrames(frames);
    	mProgram 		= Shaders.getInstance().addProgram(DEF_PROGRAM);
    	visible			= true;
    	activeFrame 	= 0;
    }
    
    /**
     * Base constructor
     */
    public SceneElement() {
    	
    	this(1);
    }
    
    
    /**
     * Base constructor
     */
    public SceneElement(int frames, String program) {
    	
    	this(frames);
    	mProgram = Shaders.getInstance().addProgram(program);
    }
     
    
    /**
     * Base constructor
     */
    public SceneElement(String program) {
    	
    	this(1, program);
    }
    
    
    /**
     * Initialize vertex byte buffer for shape coordinates
     */
    protected FloatBuffer createVertexBuffer(float[] coords) {
    	
        ByteBuffer bb = ByteBuffer.allocateDirect( coords.length * 4);	// 4 bytes per float
        bb.order(ByteOrder.nativeOrder());
        FloatBuffer vertexBuffer = bb.asFloatBuffer();
        vertexBuffer.put(coords);
        vertexBuffer.position(0);
        return vertexBuffer;
    }
    
    
    /**
     * Initialize byte buffer for the draw list
     */
    private ShortBuffer initializeDrawList(short[] drawOrder) {
    	
        ByteBuffer dlb = ByteBuffer.allocateDirect(drawOrder.length * 2);	// 2 bytes per short
        dlb.order(ByteOrder.nativeOrder());
        ShortBuffer drawListBuffer = dlb.asShortBuffer();
        drawListBuffer.put(drawOrder);
        drawListBuffer.position(0);
        return drawListBuffer;
    }
     
    
    /**
     * Apply the base transformations like, position, rotation and size.
     * 
     * @param mModelMatrix - model matrix that represents the transformations
     */
    public void transform(float[] mModelMatrix) {
    	
    	Matrix.setIdentityM(mModelMatrix, 0);
    	Matrix.translateM(mModelMatrix, 0, position.getX(), position.getY(), 0.0f);
        Matrix.rotateM(mModelMatrix, 0, rotation.getZ(), 0.0f, 0.0f, 1.0f); 
        Matrix.scaleM(mModelMatrix, 0, size, size, 1.0f);
    }
    

    /**
     * Renders the shape
     * 
     * @param mMVPMatrix - combination of the model/view/projection matrices. This data
     * is the input of the vertex shader.
     */
	public void render(float[] mMVPMatrix) {
		
		if (isVisible()) {
			
			// Add program to OpenGL environment
	        GLES20.glUseProgram(mProgram);
	        
	        // bind the data
	        bindVertexShader(mMVPMatrix);
	        bindFragmentShader();
	
	        // Draw the shape
	        GLES20.glDrawElements (
	        		GLES20.GL_TRIANGLES, 
	        		drawOrder.length,
	                GLES20.GL_UNSIGNED_SHORT, 
	                drawListBuffer
	        );
		}
	}
	
	/**
	 * Bind vertex shader atrributes, varyings and uniforms
	 */
	private void bindVertexShader(float[] mMVPMatrix) {
		
		// get handle to vertex shader's vPosition member
        int mPositionHandle = GLES20.glGetAttribLocation(mProgram, "vPosition");

        // Enable a handle to the triangle vertices
        GLES20.glEnableVertexAttribArray(mPositionHandle);

        // Prepare the triangle coordinate data
        GLES20.glVertexAttribPointer(
        		mPositionHandle, 
        		COORDS_PER_VERTEX,
        		GLES20.GL_FLOAT,
        		false,
        		vertexStride, 
        		vertexBuffer
        );
        
        // Pass the MVP data into the shader
        int mMVPMatrixHandle=0;
        GLES20.glUniformMatrix4fv(mMVPMatrixHandle, 1, false, mMVPMatrix, 0);
        
	}
	
	
	/**
	 * Bind fragment shader atrributes, varyings and uniforms
	 */
	protected void bindFragmentShader() {
		
		// get handle to fragment shader's vColor member
        int mColorHandle = GLES20.glGetUniformLocation(mProgram, "vColor");

        // Set color for drawing the triangle
        GLES20.glUniform4fv(mColorHandle, 1, color.getArrayData(), 0);
        
        // bind texture
        int mTextureUniformHandle    = GLES20.glGetUniformLocation( mProgram, "u_Texture");
        int mTextureCoordinateHandle = GLES20.glGetAttribLocation ( mProgram, "a_TexCoordinate");
        
        // Pass in the texture coordinate information
        textureFrames.getTextureBuffer(activeFrame).position(0);
        GLES20.glVertexAttribPointer(
        		mTextureCoordinateHandle, 
        		2, 
        		GLES20.GL_FLOAT, 
        		false, 
        		0, 
        		textureFrames.getTextureBuffer(activeFrame)
        );
        GLES20.glEnableVertexAttribArray(mTextureCoordinateHandle);
     
        // Set the active texture unit to texture unit 0.
        GLES20.glActiveTexture(GLES20.GL_TEXTURE0);
     
        // Bind the texture to this unit.
        GLES20.glBindTexture(GLES20.GL_TEXTURE_2D, mTexture);
     
        // Tell the texture uniform sampler to use this texture in the shader by binding to texture unit 0.
        GLES20.glUniform1i(mTextureUniformHandle, 0);
	}
	
	
	/**
	 * Set the 2D coords position of this scene element
	 */
	public void setPosition2D(float x, float y) {
		position.setX(x);
		position.setY(y);
	}
	
	
	public void setColor(float r, float g, float b) {
		this.color.set(r, g, b);
	}
	
	public void setColor(ColorRGBA color) {
		this.color.set(color);
	}
	
	
	public void setAlpha(float a) {
		this.color.setAlpha(a);
	}
	
	public float getSize() {
		return size;
	}


	public void setSize(float size) {
		this.size = size;
	}
	
	public Tuple3f getRotation() {
		return rotation;
	}

	public int getTexture() {
		return mTexture;
	}

	public void setTexture(int id) {
		this.mTexture = id;
	}

	public void setPosition(float x, float y, float z) {
		this.position.set(x, y, z);
	}
	
	
	public Tuple3f getPosition() {
		return position;
	}
	
	
	public boolean isVisible() {
		return visible;
	}

	public void setVisible(boolean visible) {
		this.visible = visible;
	}


	public void move2D(float incX, float incY) {
		position.setX(position.getX() + incX);
		position.setY(position.getY() + incY);
	}

	public void update() {
		
	}
	
}
