package org.shadoware.eggskeeper.engine;

import java.io.IOException;
import java.util.HashMap;
import java.util.Map;

import org.shadoware.eggskeeper.engine.util.GameResources;

import android.content.res.AssetManager;
import android.opengl.GLES20;


public class Shaders {
	
	private final static String VERTEX_SFX 	= ".vs";
	private final static String FRAGMENT_SFX 	= ".fs";
	
	private final static String SHADERS_FOLDER = "shaders";
	
	private final static Shaders 	INSTANCE = new Shaders();		// singleton instance
	
	
	private final Map<String, Integer> programs		= new HashMap<String, Integer>();
	private final Map<String, Integer> vertexShaders	= new HashMap<String, Integer>();
	private final Map<String, Integer> fragmentShaders	= new HashMap<String, Integer>();

	
	/**
	 * private constructor
	 */
	private Shaders() {
		
	}
	
	
	/**
	 * Returns the associated compiled and linked program
	 */
	public int getProgram(String name) {
		return programs.get(name);
	}
	
	
	/**
	 * Loadd all shader files
	 */
	public void loadShaders(AssetManager assetManager) {
		
		String sName;
		String sCode;
		
		clearAll();		// clear all the shader index
		
		try {
			String[] fileNames = GameResources.getDirectoryFiles(SHADERS_FOLDER, assetManager);
			
			for(String fileName : fileNames) {
				sName = GameResources.getFileBaseName(fileName);
				sCode = GameResources.readTextFile(fileName, assetManager);
				addShader(sName, sCode, shaderType(fileName));
			}
		} 
		catch (IOException e) {
			
		}
	}
	
	
	/**
	 * Clears the data in all maps.
	 * 
	 * When reloading shaders in onCreate method, the inices of shaders get lost, but we still storing
	 * the indices in our maps, so we need to use this method every time before loading the shaders.
	 */
	private void clearAll() {
		this.programs.clear();
		this.vertexShaders.clear();
		this.fragmentShaders.clear();
	}
	
	/**
	 * Returns the shader type in OpenGL, from the name of the shader file
	 */
	private int shaderType(String fileName) {
		
		if (fileName.contains(VERTEX_SFX)) {
			return GLES20.GL_VERTEX_SHADER;
		}
		
		else if (fileName.contains(FRAGMENT_SFX)) {
			return GLES20.GL_FRAGMENT_SHADER;
		}
		
		else return -1;
	}
	
	
	/**
	 * Get the singleton instance.
	 */
	public static Shaders getInstance() {
		return INSTANCE;
	}
	
	
	/**
	 * Gets the value of the shader name without the dot, suffix, prefix and directory bar.
	 */
	/*
	private String getBaseName(String fileName) {
		
		if (fileName.contains("/")) {
			return fileName.substring(fileName.indexOf("/")+1, fileName.indexOf("."));
		} else {
			return fileName.substring(0, fileName.indexOf("."));
		}
	}
	*/
	    
    
    /**
	 * Loads a shader returning its id
	 * 
	 * create a vertex shader type (GLES20.GL_VERTEX_SHADER)
	 * or a fragment shader type (GLES20.GL_FRAGMENT_SHADER)
	 */
	private int createShader(int type, String code) {

	    int shader = GLES20.glCreateShader(type);

	    GLES20.glShaderSource(shader, code);
	    GLES20.glCompileShader(shader);

	    return shader;
	}

	
	/**
	 * Adds a shader, only if it was not added yet.
	 */
	private void addShader(String name, String code, int type) {
		
		if (type == GLES20.GL_VERTEX_SHADER && !vertexShaders.containsKey(name))   { 
			vertexShaders.put(name, createShader(type, code));
		} 
		else if (type == GLES20.GL_FRAGMENT_SHADER && !fragmentShaders.containsKey(name)) { 
			fragmentShaders.put(name, createShader(type, code));
		}
	}
	
	
	/**
	 * Adds a program into the list, if this program does not exists yet
	 * 
	 * @param pName: program name, it should NOT exists yet
	 * @param vsName: vertex shader name, it should exists yet
	 * @param fsName: fragment shader name, it should exists yet
	 */
	public int addProgram(String pName, String vsName, String fsName) {
		
		if ( ! programs.containsKey(pName) 			&&
			   vertexShaders.containsKey(vsName) 	&&
			   fragmentShaders.containsKey(fsName) 		) {
			
			programs.put(
					pName, 
					createProgram(vertexShaders.get(vsName), fragmentShaders.get(fsName))
			);
		} 
		
		return programs.get(pName);
	}
	
	
	public int addProgram(String pName) {
		return addProgram( pName, pName, pName );
	}
	
	
	/**
	 * Remove the program, but not the linked shaders
	 */
	public void removeProgram(String name) {
		GLES20.glDeleteProgram(programs.get(name));
		programs.remove(name);
	}
	
	
	/**
	 * Create a new program from the vertex and fragment shaders
	 */
	private int createProgram(int vertexShader, int fragmentShader) {
		
		int program = GLES20.glCreateProgram(); 
		
		GLES20.glAttachShader(program, vertexShader);
		GLES20.glAttachShader(program, fragmentShader); 
		GLES20.glLinkProgram(program);
		
		return program;
	}
	
}

