package org.shadoware.eggskeeper.engine;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;


/**
 * Manages the texture coords arrays.
 * 
 * It can only handle 128 indices.
 * 
 * @author dhenar
 *
 */
public class TextureFrames {
	
	private final int 	DIM;
	private final int		NUM_FRAMES;
	private final float 	FRAME_SIZE;
	
	
	private final FloatBuffer[] texCoords;
	
	
	/**
	 * Manages the texture matrices of the animation's frames
	 * 
	 * @param dim: number of rows or columns of the texture
	 */
	public TextureFrames(int dim) {
		
		DIM 		= dim;
		NUM_FRAMES	= DIM * DIM;
		FRAME_SIZE 	= 1.0f / DIM;
		texCoords 	= new FloatBuffer[NUM_FRAMES];
		
		fillTextureMatrices(DIM, NUM_FRAMES, FRAME_SIZE, texCoords);
	}
	
	
	public FloatBuffer getTextureBuffer( int index ) throws ArrayIndexOutOfBoundsException {
		
		return texCoords[index];
	}
	
	
	/**
	 * Create all the texture matrices
	 *  
	 * @param dim : number of columns and rows
	 * @param frameSize : size of the frame in texels
	 * @param matrices : one matrix per index
	 */
	private void fillTextureMatrices( 
			int 			dim, 
			int 			numFrames, 
			float 			frameSize, 
			FloatBuffer[] 	buffers ) {
		
		byte column;
		byte row;
		
		for ( byte i=0; i<numFrames; i++ ) {
			
			column 	= (byte) ( i % dim );
			row 	= (byte) ( (i - column) / dim 	);
			
			buffers[i] = createTexturesBuffer( getSTMatrix( column, row, frameSize ) );
		}
	}
	
	
	/**
	 * Fill the texture coordinate matrix with the ST values
	 * 
	 * Rectangle (4 vertices) and 2 coordinates (s,t) per vertex... in clockwise direction
	 */
	private float[] getSTMatrix( int column, int row, float frameSize ) {
		
		float[] matrix = new float[8];
		
		float originS = column 	* frameSize;
		float originT = row 	* frameSize;
		
		// vertex 1
		matrix[0] = originS;
		matrix[1] = originT;
		
		// vertex 2
		matrix[2] = originS;
		matrix[3] = originT + frameSize;
		
		// vertex 3
		matrix[4] = originS + frameSize;
		matrix[5] = originT + frameSize;
		
		// vertex 4
		matrix[6] = originS + frameSize;
		matrix[7] = originT;
		
		return matrix;
	}
	
	
	/**
     * Create the texture coordinates Buffer
     */
    private FloatBuffer createTexturesBuffer(float[] matrix) {
    	
    	FloatBuffer mTexCoords = ByteBuffer.allocateDirect(matrix.length * 4)
    			.order(ByteOrder.nativeOrder()).asFloatBuffer();
    	
    	mTexCoords.put(matrix).position(0);
    	
    	return mTexCoords;
    }
}
