package org.shadoware.eggskeeper.engine;


public class TouchControl {
	
	private final float 	DEFAUT_SCALE_FACTOR = 0.4f;
	private final int		DIRECTION_GAP = 40;
	
	public static final byte KEY_UP 		= 0;
	public static final byte KEY_DOWN 	= 1;
	public static final byte KEY_LEFT 	= 2;
	public static final byte KEY_RIGHT 	= 3;
	public static final byte KEY_A1 		= 4;
	public static final byte KEY_A2 		= 5;
	public static final byte KEY_S1 		= 6;
	public static final byte KEY_S2 		= 7;
	public static final byte NUM_KEYS		= 8;	// sum of all keys
	
	private final 	Coordinate 	center;
	private final 	Coordinate 	touchAxis;
	private final 	Coordinate 	touchPrevious;
	private final 	Coordinate 	touchIncrement;
	private 		float 		sensibility;
	private 		float 		angle;
	private final	boolean[] 	keys;
	private			boolean		pressed;
	
	
	public TouchControl() {
		
		center			= new Coordinate();
		touchAxis 		= new Coordinate();
		touchPrevious 	= new Coordinate();
		touchIncrement	= new Coordinate();
		sensibility		= DEFAUT_SCALE_FACTOR;
		keys 			= new boolean[NUM_KEYS];
	}
	
	
	/**
	 * Update the values of the touch control
	 */
	public void updateAngle(float x, float y) {

        touchIncrement.setX(x - touchPrevious.getX());
        touchIncrement.setY(y - touchPrevious.getY());

        // reverse direction of rotation above the mid-line
        if (y > touchAxis.getY()) {
        	touchIncrement.reverseX();
        }

        // reverse direction of rotation to left of the mid-line
        if (x < touchAxis.getX()) {
        	touchIncrement.reverseY();
        }

        angle += touchIncrement.sumXY() * sensibility;
        
	    touchPrevious.set(x, y);
	}

	
	
	public boolean[] getKeys() {
		return keys;
	}


	public float getAngle() {
		return angle;
	}

	
	public float getSensibility() {
		return sensibility;
	}


	public void setSensibility(float sensibility) {
		this.sensibility = sensibility;
	}


	public float getCenterX() {
		return center.getX();
	}
	
	public float getCenterY() {
		return center.getY();
	}
	
	public boolean isPressed() {
		return pressed;
	}
	
	public void setPressed(boolean pressed) {
		this.pressed = pressed;
	}


	/**
	 * set the absolute values of the touch axis in the screen
	 */
	public void setTouchAxis(float x, float y) {
		touchAxis.set(x, y);
		touchPrevious.set(x, y);
	}
	

	/**
	 * Executed when the screen is pressed
	 * 
	 * @param x - touch event position x normalized (0.0f - 1.0f)
	 * @param y - touch event position y normalized (0.0f - 1.0f)
	 */
	public void setTouchCenter(float x, float y) {
		screenToCoords(x, y, center);
	}
	
	
	/**
	 * Converts the screen coords into OpenGL coords
	 * 
	 * @param x : screen x
	 * @param y : screen y
	 * @param point
	 */
	protected void screenToCoords(float screenX, float screenY, Coordinate point) {
		
		point.setX( 	  screenX - 0.5f  );
		point.setY( -2 * (screenY - 0.5f) );
	}
	
	
	public void resetAngle() {
		angle = 0;
	}
	
	
	/**
	 * Set all keys to false
	 */
	public void resetKeys() {
		
		for (int i=0; i<keys.length; i++) {
			keys[i] = false;
		}
	}

	
	/**
	 * Set the value of the keys when the screen is pressed
	 */
	public void updateKeysPressed(float x, float y) {
		
		keys[KEY_A1] = true;
	}
	
	
	/**
	 * Set the key values according to the position moved
	 */
	public void updateKeysMove(float x, float y) {
		
		touchIncrement.setX(x - touchPrevious.getX());
        touchIncrement.setY(y - touchPrevious.getY());

        // Y - Axis
        if (y < (touchAxis.getY() - DIRECTION_GAP)) {
        	keys[KEY_UP] 	= true;
        	keys[KEY_DOWN] 	= false;
        } 
        else if (y > (touchAxis.getY() + DIRECTION_GAP)) {
        	keys[KEY_UP] 	= false;
        	keys[KEY_DOWN] 	= true;
        }
        else {
        	keys[KEY_UP] 	= false;
        	keys[KEY_DOWN] 	= false;
        }

        // X - Axis
        if (x > (touchAxis.getX() + DIRECTION_GAP)) {
        	keys[KEY_RIGHT] = true;
        	keys[KEY_LEFT] 	= false;
        } 
        else if (x < (touchAxis.getX() - DIRECTION_GAP)) {
        	keys[KEY_RIGHT] = false;
        	keys[KEY_LEFT] 	= true;
        } 
        else {
        	keys[KEY_RIGHT] = false;
        	keys[KEY_LEFT] 	= false;
        }
       
	    touchPrevious.set(x, y);
	}
	
	
	public void updateDiscreteAngle() {
		
		if (keys[KEY_UP] && !keys[KEY_DOWN]) {
			if (keys[KEY_LEFT] && !keys[KEY_RIGHT]) {
				angle = 45.0f;
			}
			else if (!keys[KEY_LEFT] && keys[KEY_RIGHT]) {
				angle = -45.0f;
			}
			else {
				angle = 0.0f;
			}
		}
		else if (!keys[KEY_UP] && keys[KEY_DOWN]) {
			if (keys[KEY_LEFT] && !keys[KEY_RIGHT]) {
				angle = 135.0f;
			}
			else if (!keys[KEY_LEFT] && keys[KEY_RIGHT]) {
				angle = -135.0f;
			}
			else {
				angle = 180.0f;
			}
		}
		else {
			if (keys[KEY_LEFT] && !keys[KEY_RIGHT]) {
				angle = 90.0f;
			}
			else if (!keys[KEY_LEFT] && keys[KEY_RIGHT]) {
				angle = -90.0f;
			}
		}
	}
	
	
	/**
	 * For debug purposes
	 */
	public void logKeys() {
		for (int i=0; i<keys.length; i++) {
			if (keys[i]) {
				System.out.print("1" + " ");
			}
			else {
				System.out.print("0" + " ");
			}
		}
		System.out.println("");
	}
}


class Coordinate {
	
	private float x;
	private float y;
	
	
	public Coordinate() {
		
	}
	
	public Coordinate(float x, float y) {
		this.x = x;
		this.y = y;
	}

	public float getX() {
		return x;
	}
	
	public void setX(float x) {
		this.x = x;
	}
	
	public float getY() {
		return y;
	}
	
	public void setY(float y) {
		this.y = y;
	}
	
	public void set(float x, float y) {
		this.x = x;
		this.y = y;
	}
	
	public void reverseX() {
		x = -x;
	}
	
	public void reverseY() {
		y = -y;
	}
	
	public float sumXY() {
		return x + y;
	}
	
}